 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% File: This file perform data rejection of outliers according to
% Chauvenet's criterion on a large data set, with zero values removed %'
% Author: Meike Vogt & Scott Doney
% Date: 18/11/2011
% Input: Tab separated txt file
% Output: Rejected data according to Chauvenet, Plots of data distribution
% Remarks: Analysis both performed for original and log-normalized data
% Remarks: Look for comment %!!!FLAG THOSE LINES!!! to identify which lines
% to flag as outliers in your original data file (occurs twice for
% original and log-normalized exclusion procedure, choose appropriate)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Part zero: Clean your memory & close all figures
%------------------------------------------------------

clear all
close all

% Part one: Read a tab separated txt file generated from an
% Excel file in Linux - read your own file there
%------------------------------------------------------------------------

dd = importdata('yourfile.txt');

% View headers of each column

%dd.colheaders % comment out if your file has no headers
dd.textdata

% Load biomass data into variable data_bio

n = 10 % number of the column containing biomass values, modify

% Case 1: Files with column headers

data_bio=[dd.data(:,n)]; %for files with column headers

%Case 2: Files without column headers

data_bio=[dd(:,n)]; %for files without column header

% Part two - Analyse data and remove zero entries
% ------------------------------------------------------------

% compute length, mean, std, min, max of biomass data

data_length=length(data_bio);%
data_mean = mean(data_bio); %
data_standard = std(data_bio); %

% remove zero entries

data_zeros=find(data_bio==0.0);
data_nonzeros=find(data_bio>0.0);

data_bio2 = data_bio(data_nonzeros);

% percentage of non-zero

data_percent_nonzero = length(data_bio2)/data_length *100 %

% compute length, mean, std, min max of non-zero data

data_length2=length(data_bio2); %
data_mean2 =mean(data_bio2); %
data_standard2 = std(data_bio2); %
data_max2 = max(data_bio2); %
data_min2 = min(data_bio2); %


% Part three - Identify outliers using Chauvenets criterion
% -----------------------------------------------------------

% Z-score data and compute two-sided Z-score for Chauvenets criteria

data_probability = 1/(2*length(data_nonzeros)); %
data_zscore = (data_bio2 - data_mean2)/(data_standard2);
data_ptest = 1 - data_probability/2;
zc=norminv(data_ptest, 0, 1)

% Hence, reject data with biomass > std*zc

data_limit = zc * data_standard2;
data_cv = data_bio2( data_zscore >= -zc & data_zscore <= zc );
data_cvlength = length(data_cv);
index_rejected = find(data_zscore > zc | data_zscore < -zc);
%!!! index_rejected: these are the indices of the rejected values in your data vector
data_rejected =  data_bio2(data_zscore > zc | data_zscore < -zc);
%!!! data_rejected: these are the biomass values of the rejected values in your
%data vector
index_rejected_original = data_nonzeros(index_rejected); %!!!FLAG THOSE LINES!!!
biomass_rejected_original = data_bio(index_rejected_original);
%!!!index/biomass_rejected_original: these are the lines/biomasses
%of your original data file that need to be flagged

% percent of data rejected by Chavenets criterion

data_percent_rejected = (1- data_cvlength/length(data_bio2))* 100 

% compute histogram using linear bin-size
[M,Y]=hist(data_bio2,1000);
[M_cv]=hist(data_cv,Y);

% Part four - Plot figures of rejected/non-rejected data and visualize distribution
%--------------------------------------------------------------------------------------

figure(1)
subplot(2,3,1)
semilogy(Y,M,'or',Y,M_cv,'ob')
title('Biomass Histogram your PFT')
ylabel('Occurrence per bin')
xlabel('Biomass concentration')

subplot(2,3,2)
loglog(Y,M,'or',Y,M_cv,'ob')
title('Biomass Histogram your PFT') 
ylabel('Occurrence per bin')
xlabel('Biomass concentration')

subplot(2,3,3)
normplot(data_bio2)
title('Your data: Normal Probability Plot')
ylabel('Cumulative Probability')
xlabel('Biomass concentration')

% Part five - Check for normality of data
%------------------------------------------------------------------------

% Do a Lilliefors-Test to check normality of the data against
% null hypothesis 'data are normally distributed'

[H, P, L, C] = lillietest(data_bio2)
%H =0 means: data are normally distributed
% H=1 means: data are not normally distributed

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Part six - Repeat analysis for log normalized non-zero data
% Remark: Choose this if your data is not normally distributed!
% Remark: Compare results for log-normalized data to results for
% raw data, may be data from same cruise etc.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% log normalize data (with log-10)

logdata = log10(data_bio2);

% Z-score data and compute two-sided Z-score for Chauvenets criteria

z_logdata = (logdata - mean(logdata))/std(logdata);
logdata_ptest = 1 - 1/(2*length(data_bio2))/2;
logdata_zc=norminv(logdata_ptest,0,1);
logdata_cv = logdata( z_logdata >= -logdata_zc & z_logdata <= logdata_zc );
nlogdata_cv = length(logdata_cv);
i_rejected = find(z_logdata > logdata_zc | z_logdata < -logdata_zc);
%!!! these are the indices of the rejected values in your
%log-normalized data vector
logdata_rejected =  logdata(z_logdata > logdata_zc | z_logdata < -logdata_zc);
%!!! these are the biomass values of the rejected values in your
%log-normalized data vector
i_rejected_original = data_nonzeros(i_rejected); %!!!FLAG THOSE LINES!!!
logdata_rejected_original = data_bio(i_rejected_original);
logdata_rejected_original_10 = 10.^logdata_rejected;
%!!!i/logdata_rejected_original(_10): these are the lines/biomasses
%of your original data file that need to be flagged

% Perform a Lilliefors Test to check whether your log-transformed
% data is normally distributed

[H, P, L, C] = lillietest(logdata);
% H = 0, data is normally distributed
% H = 1, data is not normally distributed
% Remark: Contact a member of the MAREDAT steering committee, if the latter is the case


% compute histogram using linear bin-size

[M,Y]=hist(logdata,1000);
[M_cv]=hist(logdata_cv,Y);

% add figures for the normalized data

subplot(2,3,4)
plot(Y,M,'or',Y,M_cv,'ob')
title(' Biomass Histogram your PFT; log-10 normalized data') 
ylabel('Occurrence per bin')
xlabel('log-10 normalized biomass concentration')

subplot(2,3,5)
normplot(logdata)
title('Your data: Normal Probability Plot')
ylabel('Cumulative Probability')
xlabel('log-10 normalized biomass concentration')
